@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, Martin Lund
+ * Copyright (c) 2017-2022  Martin Lund
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,11 +40,16 @@
 #include <errno.h>
 #include "tcp.h"
 #include "error.h"
+#include <fcntl.h>
 
 int tcp_connect(void *data, const char *address, int port, const char *name, int timeout)
 {
     struct sockaddr_in server_address;
     struct hostent *host;
+    struct timeval tv;
+    int result, opt;
+    fd_set wait_set;
+    socklen_t len;
 
     tcp_data_t *tcp_data = (tcp_data_t *) data;
 
@@ -55,6 +60,19 @@ int tcp_connect(void *data, const char *address, int port, const char *name, int
         return -1;
     }
 
+    // Get socket flags
+    if ((opt = fcntl(tcp_data->server_socket, F_GETFL, NULL)) < 0)
+    {
+      error_printf("%s\n", strerror(errno));
+      return -1;
+    }
+    // Set socket non-blocking
+    if ((fcntl(tcp_data->server_socket, F_SETFL, opt | O_NONBLOCK)) < 0)
+    {
+      error_printf("%s\n", strerror(errno));
+      return -1;
+    }
+
     // Construct the server address structure
     memset(&server_address, 0, sizeof(server_address));
     server_address.sin_family      = AF_INET;
@@ -76,12 +94,56 @@ int tcp_connect(void *data, const char *address, int port, const char *name, int
         memcpy(&server_address.sin_addr, host->h_addr, sizeof(server_address.sin_addr));
     }
 
+    tv.tv_sec = timeout / 1000;
+    tv.tv_usec = (timeout % 1000) * 1000;
+
     // Establish connection to server
-    if (connect(tcp_data->server_socket, (struct sockaddr *) &server_address, sizeof(server_address)) < 0)
+    result = connect(tcp_data->server_socket, (struct sockaddr *) &server_address, sizeof(server_address));
+    if (result < 0)
     {
-        error_printf("connect() call failed\n");
+      if (errno == EINPROGRESS)
+      {
+        FD_ZERO(&wait_set);
+        FD_SET(tcp_data->server_socket, &wait_set);
+        // Wait for socket to be writable up to timeout duration
+        result = select(tcp_data->server_socket + 1, NULL, &wait_set, NULL, &tv);
+      }
+    }
+    else
+    {
+      result = 1;
+    }
+
+    // Reset socket flags
+    if (fcntl(tcp_data->server_socket, F_SETFL, opt) < 0)
+    {
+      error_printf("%s\n", strerror(errno));
+      close(tcp_data->server_socket);
+      return -1;
+    }
+
+    if (result < 0)
+    {
+      error_printf("connect() call failed\n");
+      close(tcp_data->server_socket);
+      return -1;
+    }
+    else if (result == 0)
+    {
+      error_printf("connect() call timed out\n");
+      close(tcp_data->server_socket);
+      return -1;
+    }
+    else
+    {
+      // Check for socket errors
+      len = sizeof(opt);
+      if (getsockopt(tcp_data->server_socket, SOL_SOCKET, SO_ERROR, &opt, &len) != 0)
+      {
+        error_printf("%s\n", strerror(errno));
         close(tcp_data->server_socket);
         return -1;
+      }
     }
 
     return 0;
@@ -112,15 +174,25 @@ int tcp_send(void *data, const char *message, int length, int timeout)
     FD_ZERO(&wdfs);
     FD_SET(tcp_data->server_socket, &wdfs);
 
+    // Wait for socket to be writable
     status = select(tcp_data->server_socket + 1, NULL, &wdfs, NULL, &tv);
     if (status == -1)
+    {
+        error_printf("%s\n", strerror(errno));
         return -1;
+    }
     else if (status)
     {
         // Send until all data is sent
         do
         {
             n = send(tcp_data->server_socket, message + n, length, 0);
+            if (n < 0)
+            {
+                error_printf("%s\n", strerror(errno));
+                return -1;
+            }
+
             length -= n;
             bytes_sent += n;
         }
@@ -129,7 +201,9 @@ int tcp_send(void *data, const char *message, int length, int timeout)
         return bytes_sent;
     }
     else
+    {
         error_printf("Timeout\n");
+    }
 
     return -1;
 }
@@ -150,6 +224,7 @@ static int tcp_receive_(void *data, char *message, int length, int timeout, int
     FD_ZERO(&rdfs);
     FD_SET(tcp_data->server_socket, &rdfs);
 
+    // Wait for socket to be readable
     status = select(tcp_data->server_socket + 1, &rdfs, NULL, NULL, &tv);
     if (status == -1)
         return -1;
@@ -169,10 +244,11 @@ static int tcp_receive_(void *data, char *message, int length, int timeout, int
         return bytes_received;
     }
     else
+    {
         error_printf("Timeout\n");
+    }
 
     return -1;
-
 }
 
 int tcp_receive(void *data, char *message, int length, int timeout)
